//	<><><><><><><><><><><><><>  Plot3DZ.cpp  <><><><><><><><><><><><><><> */
//		
// ----------------------------------------------------
// (C) Copyright Clark Dailey, Inc. 2006
//
// Implementation of Plot3DZ C++ class
//
// ----------------------------------------------------

#include "stdafx.h"	// precompiled header
#include <stdio.h>
#include "Plot3DZ.h"
#include "SurfX3D.h"


// ------------------------------------------------------------
Plot3DZ::Plot3DZ()
{
	// use default values
	m_is_color_xyz = 1;	// x,y,z color
	DefInputs();	
	m_pPlotCalc     = new PlotCalc;
	m_pRPlotCalc    = new PlotCalc;
	m_pGPlotCalc    = new PlotCalc;
	m_pBPlotCalc    = new PlotCalc;
	SetIniHeader("Explicit");
}

// ------------------------------------------------------------
Plot3DZ::~Plot3DZ()
{
	delete m_pPlotCalc;
	delete m_pRPlotCalc;
	delete m_pGPlotCalc;
	delete m_pBPlotCalc;
}

// ------------------------------------------------------------
double Plot3DZ::Zfunc(double x,double y)
{
	double z = m_pPlotCalc->Calc(x,y);
	if (!m_limit_z) return(z);
	// limit z values to user defined
	if (z < m_zmin) z = m_zmin;
	if (z > m_zmax) z = m_zmax;
	return(z);
}

// ------------------------------------------------------------
void Plot3DZ::Reset(void)
{
	ResetT();
	InitBase();
	DefInputs();
}

// ------------------------------------------------------------
// does the actual work of plotting
// returns: 0=ok, 1=user abort, 2=not enough memory to plot
int Plot3DZ::Plot(CWnd* pWnd)
{
	time_t begTime, endTime;
	double xmin, xmax, ymin, ymax, zmin, zmax;

	SetWnd(pWnd);

	theApp.SetStatusString("Render Begin");
	time(&begTime);

	// save value since base class modifies them
	xmin = m_xmin;
	xmax = m_xmax;
	ymin = m_ymin;
	ymax = m_ymax;
	zmin = m_zmin;
	zmax = m_zmax;

	m_umin = m_xmin;
	m_umax = m_xmax;
	m_vmin = m_ymin;
	m_vmax = m_ymax;

	if (InitPlot()) return(2);

	WinBegPlot();

	// plot color pixels
	PlotColorPixels();

	// Draw U Lines
	DrawULines();

	// Draw V Lines
	DrawVLines();

	// draw axis
	DrawAxis();

	WinEndPlot();
	time(&endTime);

	// restore values
	m_xmin = xmin;
	m_xmax = xmax;
	m_ymin = ymin;
	m_ymax = ymax;
	m_zmin = zmin;
	m_zmax = zmax;

	// calculate render time
	unsigned minutes, seconds;
	CalcDiffTime(begTime,endTime,&minutes,&seconds);

	// show rendering time
	CString sRenderTime;
	sRenderTime.Format("Render Complete    %u:%02u (m:s)", minutes, seconds);
	theApp.SetStatusString(sRenderTime);

	return(0);
}

// ------------------------------------------------------------
// default user inputs
void Plot3DZ::DefInputs()
{
	// initialize plot equation
	m_sSurfaceName  = "Untitled";
	m_zPlotEquation = "sin(x*y/5)";
	m_rPlotEquation = "(sin(6*atan2(y,x))+1)*50";
	m_gPlotEquation = "(cos(6*atan2(y,x))+1)*50";
	m_bPlotEquation = "0";
	m_sC1 = "1";
	m_sC2 = "1";
	m_sC3 = "1";
	m_sC4 = "1";
	m_sC5 = "1";
	m_sC6 = "1";
	m_sC7 = "1";
	m_sC8 = "1";
}

// ------------------------------------------------------------
int  Plot3DZ::InitEquations()
{
	int zcol, rcol, gcol, bcol;

	// check input parameters
	if (m_xmin >= m_xmax)
	{
		m_is_equation_ok = 0;
		theApp.SetStatusString("Error: Xmin >= Xmax");
		return(1);
	}
	if (m_ymin >= m_ymax)
	{
		m_is_equation_ok = 0;
		theApp.SetStatusString("Error: Ymin >= Ymax");
		return(1);
	}
	if (m_zmin >= m_zmax)
	{
		m_is_equation_ok = 0;
		theApp.SetStatusString("Error: Zmin >= Zmax");
		return(1);
	}

	// load and check equations
 	zcol = m_pPlotCalc->LoadProgram("Z(x,y)",(LPCTSTR)m_zPlotEquation,0,
				(LPCTSTR)m_sC1,(LPCTSTR)m_sC2,(LPCTSTR)m_sC3,(LPCTSTR)m_sC4,
				(LPCTSTR)m_sC5,(LPCTSTR)m_sC6,(LPCTSTR)m_sC7,(LPCTSTR)m_sC8);

 	rcol = m_pRPlotCalc->LoadProgram("R(x,y,z)",(LPCTSTR)m_rPlotEquation,1,
			(LPCTSTR)m_sC1,(LPCTSTR)m_sC2,(LPCTSTR)m_sC3,(LPCTSTR)m_sC4,
			(LPCTSTR)m_sC5,(LPCTSTR)m_sC6,(LPCTSTR)m_sC7,(LPCTSTR)m_sC8);

 	gcol = m_pGPlotCalc->LoadProgram("G(x,y,z)",(LPCTSTR)m_gPlotEquation,1,
			(LPCTSTR)m_sC1,(LPCTSTR)m_sC2,(LPCTSTR)m_sC3,(LPCTSTR)m_sC4,
			(LPCTSTR)m_sC5,(LPCTSTR)m_sC6,(LPCTSTR)m_sC7,(LPCTSTR)m_sC8);

 	bcol = m_pBPlotCalc->LoadProgram("B(x,y,z)",(LPCTSTR)m_bPlotEquation,1,
			(LPCTSTR)m_sC1,(LPCTSTR)m_sC2,(LPCTSTR)m_sC3,(LPCTSTR)m_sC4,
			(LPCTSTR)m_sC5,(LPCTSTR)m_sC6,(LPCTSTR)m_sC7,(LPCTSTR)m_sC8);

	m_is_equation_ok = (zcol | rcol | gcol | bcol) ? 0 : 1;

	// show status string
	if (zcol)
	 	m_sEquationStatus = m_pPlotCalc->GetErrString();
	else if (rcol)
	 	m_sEquationStatus = m_pRPlotCalc->GetErrString();
	else if (gcol)
	 	m_sEquationStatus = m_pGPlotCalc->GetErrString();
	else
	 	m_sEquationStatus = m_pBPlotCalc->GetErrString();

	if (!m_is_equation_ok) theApp.SetStatusString(m_sEquationStatus);
	return(m_is_equation_ok?0:1);
}

// --------------------------------------------------------------
// The mathematical equation used to define the "color" of the surface.
long Plot3DZ::Color(double x,double y,double z)
{
	#define NCOLORS	6
	#define NSHADES	(NCOLORS*2)
	static long ColorValues[NCOLORS] = 
	{   //   R   G   B
		RGB(255,000,000),	// red
		RGB(255,128,000),	// orange
		RGB(200,200,000),	// yellow
		RGB(000,160,000),	// green
		RGB(000,000,200),	// blue
		RGB(128,000,128)	// violet
	};
	int iz;

	iz = abs((int)(2.*NSHADES*(z-m_zmin)/(m_zmax-m_zmin)));
	if ((iz & 1)==0) return( PLOT_COLOR_BLACK ); // black band between each color
	iz = iz % (NSHADES-1);
	return( ColorValues[iz] );
}

// ------------------------------------------------------------
// returns: 0=ok, 1=can't open file
int  Plot3DZ::ReadFile()
{
	if (!DoesFileExist(m_FilePath)) return(1);

	// surface definition
	m_sSurfaceName  = GetIniStr   ("Name",          	"Untitled"    );
	m_zPlotEquation = GetIniStr   ("PlotEquation", 	    "sin(x*y/5)"         );
	m_rPlotEquation = GetIniStr   ("RPlotEquation", 	"(sin(z*c8)+1)*50"    );
	m_gPlotEquation = GetIniStr   ("GPlotEquation", 	"(cos(z*c8)+1)*50"    );
	m_bPlotEquation = GetIniStr   ("BPlotEquation", 	"0"    );
	m_sC1           = GetIniStr   ("C1",	          	"1"    );
	m_sC2           = GetIniStr   ("C2",	          	"1"    );
	m_sC3           = GetIniStr   ("C3",	          	"1"    );
	m_sC4           = GetIniStr   ("C4",	          	"1"    );
	m_sC5           = GetIniStr   ("C5",	          	"1"    );
	m_sC6           = GetIniStr   ("C6",	          	"1"    );
	m_sC7           = GetIniStr   ("C7",	          	"1"    );
	m_sC8           = GetIniStr   ("C8",	          	"1"    );

	// common parameters
	m_zrotate       = GetIniDouble("ZRotate",       	30.0  );
	m_xytilt        = GetIniDouble("XYTilt",        	60.0  );
	m_scale         = GetIniDouble("PlotScale",        100.0  );
	m_line_color    = GetIniLong  ("LineColor",     	 DEF_LINE_COLOR    );
	m_back_color    = GetIniLong  ("BackColor",          DEF_BKGND_COLOR   );
	m_is_color      = GetIniLong  ("ShowColor",     	 1    );
	m_remove_hidden = GetIniLong  ("RemoveHidden",  	 1    );
	m_show_axis     = GetIniLong  ("ShowAxis",      	 0    );

	// specific parameters
	m_xmin          = GetIniDouble("XMin",              -10.  );
	m_xmax          = GetIniDouble("XMax",               10.  );
	m_ymin          = GetIniDouble("YMin",              -10.  );
 	m_ymax          = GetIniDouble("YMax",               10.  );
	m_zmin          = GetIniDouble("ZMin",              -10.  );
 	m_zmax          = GetIniDouble("ZMax",               10.  );
	m_nulines       = GetIniLong  ("NXLines",            30   );
	m_nvlines       = GetIniLong  ("NYLines",            30   );
	m_draw_ulines   = GetIniLong  ("ShowX",    			 1    );
	m_draw_vlines   = GetIniLong  ("ShowY",		    	 1    );
	m_limit_z       = GetIniLong  ("LimitZ",  	    	 0    );

	return(0);
}

// ------------------------------------------------------------
int  Plot3DZ::WriteFile()
{
	// surface definition
	SetIniStr   ("Name",          m_sSurfaceName);
 	SetIniStr   ("PlotEquation",  m_zPlotEquation);
 	SetIniStr   ("RPlotEquation", m_rPlotEquation);
 	SetIniStr   ("GPlotEquation", m_gPlotEquation);
 	SetIniStr   ("BPlotEquation", m_bPlotEquation);
	SetIniStr   ("C1",	          m_sC1);
	SetIniStr   ("C2",	          m_sC2);
	SetIniStr   ("C3",	          m_sC3);
	SetIniStr   ("C4",	          m_sC4);
	SetIniStr   ("C5",	          m_sC5);
	SetIniStr   ("C6",	          m_sC6);
	SetIniStr   ("C7",	          m_sC7);
	SetIniStr   ("C8",	          m_sC8);

	// common parameters
	SetIniDouble("ZRotate",       m_zrotate);
	SetIniDouble("XYTilt",        m_xytilt);
	SetIniDouble("PlotScale",     m_scale);
	SetIniLong  ("LineColor",     m_line_color);
	SetIniLong  ("BackColor",     m_back_color);
	SetIniLong  ("ShowColor",     m_is_color);
	SetIniLong  ("RemoveHidden",  m_remove_hidden);
 	SetIniLong  ("ShowAxis",      m_show_axis);

	// specific parameters
	SetIniDouble("XMin",          m_xmin);
	SetIniDouble("XMax",          m_xmax);
	SetIniDouble("YMin",          m_ymin);
	SetIniDouble("YMax",          m_ymax);
	SetIniDouble("ZMin",          m_zmin);
	SetIniDouble("ZMax",          m_zmax);
	SetIniLong  ("NXLines",       m_nulines);
	SetIniLong  ("NYLines",       m_nvlines);
	SetIniLong  ("ShowX",         m_draw_ulines);
	SetIniLong  ("ShowY",         m_draw_vlines);
	SetIniLong  ("LimitZ",        m_limit_z);

	return(0);
}

/*	<><><><><><><><><><><><><>  Plot3DZ.cpp  <><><><><><><><><><><><><><> */


















